% This function computes the negative loglikelihood function for the joint population size
% and density estimation model as stated in Equation (17) of [1] for a given
% set of parameters and measurements.  For this function we assume that 
% the individuals' arrival/departure time distribution (actual or approximate) are known, 
% but not the actual times.
%
% Input:
% parameters is a (K+1)-element vector containing the K alpha parameters of the
% gamma distributions modeling the contact rates of every location. The (K+1)th
% element is the beta parameter of the gamma distributions.
% allIndTau is a matrix where element ki is equal to the summation of the overlap
% times of each agent's presence in location k and the presence of the i'th individual 
% (from the sample of the individual's arrival/departure times from allEntDepTimes) in the event. 
% The matrix has K rows. The number of columns is equal to the samples of the individuals' arrival/departure
% times from allEntDepTimes.
% contMat is an SxMxK matrix where element ijk is equal to the number of times
% that agent j contacts individual i in location k, for 1 <= i <= S, 1 <= j <= M, and 1 <= l <= K.
% Psi is used to compute a part of the loglikelihood function given in Equation (17) of
% [1]. In particular, it is a matrix with S rows. The number of columns is equal to the
% number of samples of the arrival/departure times from allEntDepTimes. It contains
% sum_{l=1}^{K}sum_{j=1}^{M}log(k_{ij}^{(l)})t_{at, dt}^{j,(l)} where 1 <=i <=S
% is the id of the discovered individual and (at, dt) is a sample
% arrival/departure time from allEntDepTimes.
%
% Output:
% NhatValue is the jointly estimated population size for the given set of parameters and
% measurements
% logLike is the estimated negative log likelihood value given the set of
% parameters and measurements
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/
% [2]: "Opportunistic Sampling for Joint Population Size and Density
% Estimation: Supplementary Notes", F. M. Naini, O. Dousse, P. Thiran,
% and M. Vetterli, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

function [logLike, NhatValue] = JointPopDensEstLogLikeKnownDistribution(parameters, allIndTau, contMat, Psi)

[S, M, K] = size(contMat);
parameters = reshape(parameters, [1, length(parameters)]);

% the location-dependent alpha and beta parameters
alphaValues = parameters(1:end - 1);
betaValue = parameters(end);

% computing the probability of discovering an individual by any agent for
% the given set of parameters
sumDetMat = reshape(sum(contMat, 2), [S, K]);
probNotDetValue = mean(exp(alphaValues * log(betaValue ./ (betaValue + allIndTau))));
probDetValue = 1 - probNotDetValue;

% the ML estimate for population size given the current
% values of alphas and beta
NhatValue = ceil(S / probDetValue);

%% estimating the loglikelihood of the undiscovered individuals
% part 1 represents the loglikelihood of nchoosek (N, S) in Equation (17) of [1]
logLikeUnObs_part1 = gammaln(NhatValue + 1) - gammaln(NhatValue - S + 1) - gammaln(S + 1);
% part 2 represents the loglikelihood of the first expection including the (N-S) exponent
% in Equation (17) of [1]
logLikeUnObs_part2 = (NhatValue - S) * log(1 - probDetValue);
if isnan(logLikeUnObs_part2)
    % in this case probDetValue=1 and NhatValue=S
    logLikeUnObs_part2 = 0;
end
logLikeUnObs = logLikeUnObs_part1 + logLikeUnObs_part2;

%% estimating the loglikelihood of the discovered individuals
% part 1 represents the loglikelihood of (prod_{l=1}^{K} beta^alpha^(l)) for all
% the S discovered individuals in Equation (17) of [1] 
logLikeObs_part1 = (sum(alphaValues) * log(betaValue)) * S;
% part 2 represents the loglikelihood of (prod_{l=1}^{K} gamma(alpha^(l)+k_i^(l))/gamma(alpha^(l))) for all
% the S discovered individuals in Equation (17) of [1]
logLikeObs_part2 = sum(sum(gammaln(repmat(alphaValues, [S 1]) + sumDetMat) - repmat(gammaln(alphaValues), [S 1]), 2));
% part 3 represents the loglikelihood of the expectation of
% (prod_{l=1}^{K}prod_{j=1}^{M}t_{at,dt}^{j,(l)})^(k_ij^(l)))/((beta+sum_{j=1}^{M}t_{at,dt}^{j,(l)})^(alpha^(l)+k_i^(l)))
% for all the S discovered individuals in Equation (17) of [1]
logLikeObs_temp = (repmat(alphaValues, [S, 1]) + sumDetMat) * log(betaValue + allIndTau);
logLikeObs_part3 = sum(log(mean(exp(Psi - logLikeObs_temp), 2)));
% part 4 represents the loglikelihood of (prod_{l=1}^{K} prod_{j=1}^{M}1/(k_ij^(l)!))
% for all the S discovered individuals in Equation (17) of [1]
logLikeObs_part4 = - sum(sum(sum(gammaln(contMat + 1))));

logLikeObs = logLikeObs_part1 + logLikeObs_part2 + logLikeObs_part3 + logLikeObs_part4;

% computing the total loglikelihood
logLike = logLikeObs + logLikeUnObs;

% negating
logLike = - logLike;