% This function computes an initial starting value for the parameters of the gamma
% distribution alpha and beta for performing the optimization of the population size estimation
% model discussed in [1]
%
% Input:
% contMat is an SxM matrix where element ij is equal to the number of times
% that individual i is contacted by agent j, for 1<=i<=S and 1<=j<=M.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% allEntDepTimes is a two-column matrix containing samples of the
% arrival/departure times of the individuals to/from the event. Note that
% these samples could represent the actual distribution or an approximate
% distribution of the arrival/departure times.
%
% Output:
% alphaStart and betaStart are initial estimates of the alpha and beta parameters
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

function [alphaStart, betaStart] = parameterStart1(contMat, agentEntDepTimes, allEntDepTimes)

[S, M] = size(contMat);

% estimating commonStayTime, which is equal to time during which both
% an individual and an agent are in the event. For the individuals we use
% here the arrival/departure times in allEntDepTimes, which are either the actual
% times or drawn from the actual or approximate distribution
commonStayTime = zeros(size(allEntDepTimes, 1), M);
for indIter = 1:size(allEntDepTimes, 1)
    for agentIter = 1:M
        commonTime = max(min(allEntDepTimes(indIter, 2), agentEntDepTimes(agentIter, 2)) - max(allEntDepTimes(indIter, 1), agentEntDepTimes(agentIter, 1)), 0);
        if commonTime > 0
            commonStayTime(indIter, agentIter) = commonTime;
        end
    end
end

sumDet = sum(contMat, 2);
samplesOverlapTime = sum(commonStayTime, 2);

% estimating the moments of the number of contacts
cmiu = mean(samplesOverlapTime);
kmiu = mean(sumDet);
cmiu2 = mean(samplesOverlapTime .^ 2);
kmiu2 = mean(sumDet .^ 2);

% estimating an initial estimate of the parameters
betaStart = cmiu2 * kmiu / cmiu * 1 / (kmiu2 - kmiu - kmiu ^ 2 * cmiu2 / cmiu ^ 2);
alphaStart = kmiu * betaStart / cmiu;